# coding: utf-8
# 2021/5/18 @ tongshiwei

TEXT_SYMBOL = "[TEXT]"
FORMULA_SYMBOL = "[FORMULA]"
FIGURE_SYMBOL = "[FIGURE]"
QUES_MARK_SYMBOL = "[MARK]"
TAG_SYMBOL = "[TAG]"
SEP_SYMBOL = "[SEP]"
TEXT_BEGIN = r"[TEXT_BEGIN]"
TEXT_END = r"[TEXT_END]"
FORMULA_BEGIN = r"[FORMULA_BEGIN]"
FORMULA_END = r"[FORMULA_END]"


class Symbol(str):
    pass
