# coding: utf-8
# 2021/5/18 @ tongshiwei

from .segment import (SegmentList, TextSegment, FigureFormulaSegment, LatexFormulaSegment, FigureSegment,
                      QuesMarkSegment, Figure, TagSegment, SepSegment, seg)
